@@ -1,4 +1,4 @@
-/* WMix 3.2 -- a mixer using the OSS mixer API.
+/* WMix 3.0 -- a mixer using the OSS mixer API.
  * Copyright (C) 2000, 2001
  *	Daniel Richard G. <skunk@mit.edu>,
  *	timecop <timecop@japan.co.jp>
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 void		lr_to_vb	(float left, float right, float *volume, float *balance);
@@ -23,4 +23,4 @@ void		vb_to_lr	(float volume, float balance, float *left, float *right);
 double		get_current_time(void);
 void		add_region	(int index, int x, int y, int width, int height);
 int		check_region	(int x, int y);
-void		config_read	(void);
+void		create_pid_file	(void);
