@@ -1,4 +1,4 @@
-/* WMix 3.2 -- a mixer using the OSS mixer API.
+/* WMix 3.0 -- a mixer using the OSS mixer API.
  * Copyright (C) 2000, 2001
  *	Daniel Richard G. <skunk@mit.edu>,
  *	timecop <timecop@japan.co.jp>
@@ -15,17 +15,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <sys/types.h>
 #include <stdio.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <sys/time.h>
 
 #include "include/common.h"
@@ -40,8 +42,6 @@ typedef struct {
 } MRegion;
 MRegion mr[16];
 
-extern Config config;
-
 /* Converts separate left and right channel volumes (each in [0, 1]) to
  * volume and balance values. (Volume is in [0, 1], balance is in [-1, 1])
  */
@@ -107,61 +107,23 @@ int check_region(int x, int y)
     return (i - 1);
 }
 
-void config_read(void)
+/* handle writing PID file, silently ignore if we can't do it */
+void create_pid_file(void)
 {
+    char *home;
+    char *pid;
     FILE *fp;
-    char buf[512];
-    char *ptr;
 
-    if (config.file == NULL)
+    home = getenv("HOME");
+    if (home == NULL)
 	return;
 
-    fp = fopen(config.file, "r");
-    if (!fp)
-	return;
-
-    while (fgets(buf, 512, fp)) {
-	if ((ptr = strstr(buf, "mousewheel="))) {
-	    ptr += 11;
-	    config.mousewheel = atoi(ptr);
-	}
-	if ((ptr = strstr(buf, "scrolltext="))) {
-	    ptr += 11;
-	    config.scrolltext = atoi(ptr);
-	}
-	if ((ptr = strstr(buf, "osd="))) {
-	    ptr += 4;
-	    config.osd = atoi(ptr);
-	}
-	if ((ptr = strstr(buf, "osdcolor="))) {
-	    char *end;
-	    ptr += 9;
-	    end = strchr(ptr, '\n');
-	    ptr[end - ptr] = '\0';
-	    if (config.osd_color)
-		free(config.osd_color);
-	    config.osd_color = strdup(ptr);
-	}
-	if ((ptr = strstr(buf, "wheelstep="))) {
-	    ptr += 10;
-	    /* detect old style config */
-	    if (atoi(ptr) > 1)
-		config.scrollstep = (float)atoi(ptr) / 100.0;
-	    else
-		config.scrollstep = atof(ptr);
-	}
-	if ((ptr = strstr(buf, "wheelbtn1="))) {
-	    ptr += 10;
-	    config.wheel_button_up = atoi(ptr);
-	}
-	if ((ptr = strstr(buf, "wheelbtn2="))) {
-	    ptr += 10;
-	    config.wheel_button_down = atoi(ptr);
-	}
-        if ((ptr = strstr(buf, "lcd="))){
-	    ptr +=4;
-	    config.lcd = atoi(ptr);
-	}
+    pid = malloc(strlen(home) + 11);
+    sprintf(pid, "%s/.wmix.pid", home);
+    fp = fopen(pid, "w");
+    if (fp) {
+	fprintf(fp, "%d\n", getpid());
+	fclose(fp);
     }
-    fclose(fp);
+    free(pid);
 }
